#!/usr/bin/env python
#
# $Id: SearchResults.py,v 1.1 2002/03/11 01:05:25 dhellmann Exp $
#
# Copyright 2002 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: SearchResults.py,v $',
    'rcs_id'       : '$Id: SearchResults.py,v 1.1 2002/03/11 01:05:25 dhellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'WebAgent',
    'created'      : 'Sun, 10-Mar-2002 10:42:46 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: dhellmann $',
    'version'      : '$Revision: 1.1 $',
    'date'         : '$Date: 2002/03/11 01:05:25 $',
}

#
# Import system modules
#
from formatter import NullFormatter
import htmllib
import UserList

#
# Import Local modules
#


#
# Module
#
class SearchResultItem:
    def __init__(self, url, title='', description=''):
        self.url = url
        self.setTitle(title)
        self.setDescription(description)
        return

    def setTitle(self, title):
        self.title = title
        return

    def setDescription(self, description):
        self.description = description
        return

    def __str__(self):
        return '%s\n\t%s\n\n\t%s' % (self.title, self.url, self.description)
    

class SearchResultsList(htmllib.HTMLParser, UserList.UserList):
    """Base class for parser that knows about search engine output.

    This parser (based on HTMLParser from htmllib) knows how to
    extract information from a summary report generated by a search
    engine.  The specific knowledge of a search engine's output must
    be provided by the subclass.  This parser provides the basic
    services necessary to construct such a tool.  
    
    """

    def __init__(self):
        "Create a SearchResultsList."
        UserList.UserList.__init__(self)
        self.field_names = {}
        self.current_record = None
        self.trace = 0
        
        formatter = NullFormatter()
        htmllib.HTMLParser.__init__(self, formatter)

        self._continuation_links = []
        return

    def tuplesToDict(self, attrs):
        """Convert a sequence of tuples to a dictionary.

        The sequence should contain ('name', 'value') pairs.  The
        'name' value will be used as the key of the dictionary, and
        the 'value' will be stored as the value.  Duplicate 'name'
        values are not detected.
        """
        d = {}
        for name, value in attrs:
            d[name] = value
        return d

    def __str__(self):
        return '\n\n--------------------\n\n'.join([ str(x) for x in self ])

    def getContinuationList(self):
        return self._continuation_links
