#!/usr/bin/env python
#
# $Id: WebAgent.py,v 1.17 2002/03/16 19:06:19 dhellmann Exp $
#

"""WebAgent Base Class

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: WebAgent.py,v $',
    'rcs_id'       : '$Id: WebAgent.py,v 1.17 2002/03/16 19:06:19 dhellmann Exp $',
    'creator'      : 'Doug Hellmann <DougHellmann@bigfoot.com>',
    'project'      : 'CMFAgents',
    'created'      : 'Thu, 12-Jul-2001 10:44:08 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: dhellmann $',
    'version'      : '$Revision: 1.17 $',
    'date'         : '$Date: 2002/03/16 19:06:19 $',
}

#
# Import system modules
#
import urllib
import sys
import htmllib
import string
import pprint
import re
import UserList

#
# Import Local modules
#
from CMFFeedApp import CMFFeedApp
import DateTime
from searchengines import SearchEngines
from SearchResults import SearchResultItem, SearchResultsList

#
# Module
#

date_pattern='(?P<month>\d+)/(?P<day>\d+)/(?P<year>\d+)'
time_pattern= '(?P<hour>\d+):(?P<minute>\d+):(?P<second>\d+)\s+(?P<ampm>(AM|PM))'
date_parts_splitter=re.compile('\s*' + date_pattern + '\s+' + time_pattern).match

def date_to_datetime(input_date):
    # Example:   4/30/2001 9:18:59 AM
    match = date_parts_splitter(input_date)
    if not match:
        raise ValueError(input_date)
    parts = match.groupdict()
    dt_values = map( lambda x: int(x),
                     (parts['year'],
                      parts['month'],
                      parts['day'],
                      parts['hour'],
                      parts['minute'],
                      parts['second'],
                      )
                     )
    
    if (string.lower(parts['ampm']) == 'pm') and (dt_values[3] < 12):
        dt_values[3] = dt_values[3] + 12

    dt_values.append('EST')
    dt_values = tuple(dt_values)
    date_time = apply(DateTime.DateTime, dt_values)
    return date_time

def time_difference(start, finish):
    #
    # Define the epoch minus seconds.
    #
    epoch = (1969, 12, 31, 0, 0)
    #
    # Get the pieces of the date value but ignore
    # the timezone and seconds.
    #
    finish = finish.parts()[:-2]
    start = start.parts()[:-2]
    time_diff = map( lambda x, y: y-x,
                     start,
                     finish
                     )
    years, months, days, hours, minutes = time_diff
    if minutes < 0:
        hours = hours - 1
        minutes = minutes + 60
    if hours < 0:
        days = days - 1
        hours = hours + 24
    if days < 0:
        months = months - 1
        # rough estimate / average
        days = days + 30
    if months < 0:
        years = years - 1
        months = months + 12
        
    time_diff = years, months, days, hours, minutes
        
    return time_diff

def str_time_diff(args):
    part_names = ('year',
                  'month',
                  'day',
                  'hour',
                  'minute',
                  )
    rep = ''
    for val, name in map(None, args, part_names):
        if val:
            if val > 1:
                plural='s'
            else:
                plural=''
            rep = '%s %d %s%s' % (rep, val, name, plural)
    return string.strip(rep)

    
    
class WebAgent(CMFFeedApp):

    process_all = 1
    debug = 0

    #
    # Static settings to provide access to web input
    #

    default_config_values = {}
    default_config_values.update(CMFFeedApp.default_config_values)
    default_config_values.update({
        'URL':'',
        'SERVERS':{},
        })

    def optionHandler_a(self):
        "Process all pages."
        self.process_all = 1
        return

    def optionHandler_A(self):
        "Do not process all pages."
        self.process_all = 0
        return

    def optionHandler_debug(self):
        "Turn on debug mode and show tracebacks."
        self.debug = 1
        return

    def getAgentActions(self, folder):
        "Return a sequence of actions to take for the specified folder."
        raise RuntimeError('Not implemented')

    def processOneResult(self, parser, hit, folder):
        "Override this method to handle one of the results in the result set."
        raise RuntimeError('%s.processOneResult not implemented.' \
                           % self.__class__.__name__)

    def processResults(self, parser, folder):
        """Call self.processOneResult() for eac item in the result set.

        This method can usually be left alone, but it is here so that
        if a more efficient manner of processing an entire result set
        is available it can be used.
        """
        for hit in parser:
            self.processOneResult(parser, hit, folder)
        return

    def loadDataForFolder(self, folder):
        """Extract search terms from the title of the folder,
        and run the search.
        """
        self.statusMessage('%s:' % folder.id, 2)
        #
        # Record of what we've seen
        #
        processed_actions = []
        
        #
        # Record of what we need to look at
        #
        action_queue = []
        
        #
        # Set up queue of continuation pages to fetch
        #
        default_actions = self.getAgentActions(folder)
        for action in default_actions:
            action_queue.append(action)

        while action_queue:

            #
            # Queue management
            #
            action = action_queue[0]
            action_queue = action_queue[1:]

            processed_actions.append(action)

            self.statusMessage('Taking Action: %s' % str(action), 2)

            try:
                results = action()
            except KeyboardInterrupt:
                raise
            except Exception, msg:
            	self.statusMessage('ERROR In Action: %s' % str(action))
                self.statusMessage('ERROR: %s' % str(msg))
                if self.debug:
                    raise
                continue
            
            #
            # Process results
            #
            try:
                self.processResults(results, folder)
            except KeyboardInterrupt:
                raise
            except Exception, msg:
            	self.statusMessage('Error Processing Action: %s' % str(action))
                self.statusMessage('ERROR: %s' % str(msg))
                if self.debug:
                    raise
                continue
            
            if self.process_all:
                for continuation_page in results.getContinuationList():
                    self.statusMessage('Want to continue to %s' % continuation_page, 2)
                    if ( (continuation_page not in action_queue)
                         and
                         (continuation_page not in processed_actions)
                         ):
                        action_queue.append( continuation_page )
                        
        self.statusMessage('Processed %d actions for %s.' % \
                           (len(processed_actions),
                            folder.id
                            ),
                           2)
        return
