#!/usr/local/bin/python
#
# $Id: agent.py,v 1.5 2002/03/11 01:21:31 dhellmann Exp $
#

"""CMF Web Agent

  Retrieves a search result set and load each item into a CMF Portal.
"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: agent.py,v $',
    'rcs_id'       : '$Id: agent.py,v 1.5 2002/03/11 01:21:31 dhellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'CMFAgents',
    'created'      : 'Wed, 13-Jun-2001 07:38:43 EDT',

    #
    #  Current Information
    #
    'author'       : '$Author: dhellmann $',
    'version'      : '$Revision: 1.5 $',
    'date'         : '$Date: 2002/03/11 01:21:31 $',
}

#
# Import system modules
#

#
# Since ZctlApp.py messes with the current working directory,
# we stick that location on the sys.path so we can make
# sure we import our modules.
#
import sys, os
sys.path.append(os.getcwd())

#
# Import Local modules
#
from WebAgent import WebAgent, SearchResultsList, SearchResultItem
from searchengines import SearchEngines

#
# Module
#

class CMFWebAgent(WebAgent):

    default_config_values = {}
    default_config_values.update(WebAgent.default_config_values)

    max_title_len = 50

    def optionHandler_l(self, titleLength):
        "Set the maximum title length for display."
        self.max_title_len = int(titleLength)
        return
    
    def getAgentActions(self, folder):
        actions = []
        search_terms = folder.Description()
        if not search_terms:
            self.statusMessage('No search terms specified.', 2)
            return actions
        else:
            self.statusMessage('Search terms = "%s"' % search_terms, 2)
        for name, search_engine_factory in SearchEngines.items():
            self.statusMessage('Getting search pages from %s' % name, 2)
            search_engine = search_engine_factory()
            pages = search_engine(search_terms)
            for page in pages:
                actions.append(page)
        return actions

    def processResults(self, parser, folder):
        #
        # Reverse to put the least-likely matches
        # first.  This, in turn, makes the creation
        # times work out so that the most likely matches
        # are created last and appear closer to the
        # top of the "current news" list.
        #
        parser.reverse()
        return WebAgent.processResults(self, parser, folder)
        

    def processOneResult(self, parser, hit, folder):
        
        url = hit.url
        id = self.urlToId(url)
        title = hit.title.strip()

        # start the description with the title, in case it
        # is truncated later
        description = '<h4>%s</h4>' % title

        # Clean up the title.
        for tag in ( 'b', 'i', 'u', 'font' ):
            title = title.replace('<%s>' % tag, '')
            title = title.replace('</%s>' % tag, '')
        if len(title) > self.max_title_len:
            title = '%s ...' % title[:self.max_title_len]

        # Finish off the description, and include the url
        description = description + ('<p>%s</p>' % hit.description)
        description = description + ('<p><b>Remote Page:</b> %s</p>' % url)

        # Try to store the article
        self.addNewsItem(folder, id, title, url, description)
        return


#
# Main
#
if __name__ == '__main__':
    try:
        CMFWebAgent().run()
    except CMFWebAgent.HelpRequested:
        pass
    
