#!/usr/bin/env python
#
# $Id: __init__.py,v 1.3 2002/03/11 12:45:18 zope Exp $
#
# Copyright 2002 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Package of plugins for submitting searches to various internet
search engines.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: __init__.py,v $',
    'rcs_id'       : '$Id: __init__.py,v 1.3 2002/03/11 12:45:18 zope Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'WebAgents',
    'created'      : 'Sun, 10-Mar-2002 10:00:47 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: zope $',
    'version'      : '$Revision: 1.3 $',
    'date'         : '$Date: 2002/03/11 12:45:18 $',
}
try:
    __version__ = __rcs_info__['version'].split(' ')[1]
except:
    __version__ = '0.0'

#
# Import system modules
#
import urllib

#
# Import Local modules
#
import pluginloader

#
# Module
#

class WebAgentUrlOpener(urllib.FancyURLopener):

    #
    # Replace the user agent header string
    # used by urllib.URLopener.
    #
    version = 'SearchEngines %s' % __version__


class SearchPageBase:
    "Base class for SearchPage classes that retrieve one search URL."

    cookies = []
    
    def __init__(self, url):
        self.url = url
        return

    def __cmp__(self, other):
        return cmp(self.url, other.url)

    def __str__(self):
        return '%s(%s)' % (self.__class__.__name__, self.url) 

    def retrieveData(self):
        "Retrieve our data."
        url_opener = WebAgentUrlOpener()
        for cookie in self.cookies:
            url_opener.addheader('Cookie', '%s=%s' % cookie)
        tmpfile, dataSource  = url_opener.retrieve(self.url)
        data = open(tmpfile, 'rt').read()
        for bad_char in ( 0x99, 0x92 ):
            data.replace(chr(bad_char), '')
        return data

    def newParser(self):
        """Returns an instance of a parser that will convert the
        data in the page we retrieved to a sequence of
        search results.
        """
        raise RuntimeError('Not implemented')

    def __call__(self):
        "Returns search results list."
        data = self.retrieveData()
        parser = self.newParser()
        parser.feed(data)
        parser.close()
        return parser

    

class SearchEngineBase:

    def __call__(self, searchTermText):
        """Return a sequence of SearchPage instances for
        the searchTermText.
        """
        raise RuntimeError('Not implemented')
    

class SearchEngineLoader(pluginloader.PluginLoader):

    def __init__(self):
        pluginloader.PluginLoader.__init__(self, 'searchengines',
                                           __path__[0],
                                           None)
        return

    def addEntryPoint(self, infoDict):
        name = infoDict['name']
        #print 'Loading search engine %s' % name
        factory = infoDict['factory']
        self[name] = factory
        return
    
SearchEngines = SearchEngineLoader()

