#!/usr/bin/env python
#
# $Id: searchengines_alltheweb.py,v 1.4 2002/03/16 19:06:55 dhellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Parse AllTheWeb search results.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: searchengines_alltheweb.py,v $',
    'rcs_id'       : '$Id: searchengines_alltheweb.py,v 1.4 2002/03/16 19:06:55 dhellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'WebAgents',
    'created'      : 'Mon, 31-Dec-2001 08:20:34 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: dhellmann $',
    'version'      : '$Revision: 1.4 $',
    'date'         : '$Date: 2002/03/16 19:06:55 $',
}
try:
    __version__ = __rcs_info__['version'].split(' ')[1]
except:
    __version__ = '0.0'

#
# Import system modules
#
import pprint
import re
import string
import sys
import urllib

#
# Import Local modules
#
from SearchResults import SearchResultsList, SearchResultItem
from searchengines import SearchEngineBase, SearchPageBase


#
# Module
#

    
class AllTheWebSearchResultsList(SearchResultsList):

    def __init__(self):
        SearchResultsList.__init__(self)
        self._accumulate_data = 0
        self._catch_links = 0
        self._buffer = ''
        return

    def start_dl(self, args):
        #print 'START DL'
        self._accumulate_data = 1
        return

    def end_dl(self):
        #print 'END DL'
        self._accumulate_data = 0
        return

    def start_dt(self, args):
        #print 'START DT'
        self._catch_links = 1
        self.current_record = {}
        return

    def end_dt(self):
        self._catch_links = 0
        title = self._buffer.strip()
        self.current_record['title'] = title
        #print 'END DT (%s)' % title
        self._buffer = ''
        return

    def start_dd(self, args):
        #print 'START DD'
        return

    def end_dd(self):
        #print 'END DD'
        description = self._buffer
        self._buffer = ''
        if self.current_record:
            url = self.current_record['url']
            title = self.current_record['title']
            new_item = SearchResultItem(url, title, description)
            self.append(new_item)
            self.current_record = {}
        return

    mouse_over_re = re.compile("status='([^']+)'")
    
    def start_a(self, args):
        args = self.tuplesToDict(args)
        if self._catch_links:
            #print 'START A %s' % args['href']
            url = None
            try:
                mouse_over = args['onmouseover']
                match = self.mouse_over_re.search(mouse_over)
                if match:
                    url = match.groups()[0]
            except KeyError:
                pass
            if not url:
                url = args['href']
            self.current_record['url'] = url

        elif args.get('class', None) == 'link':
            url = AllTheWebSearchEngine.url_base + args['href']
            self._continuation_links.append(AllTheWebSearchPage(url))
            
        return

    def end_a(self):
        if self._catch_links:
            #print 'END A'
            pass
        return

    ignore_data = re.compile(r'^\[.*\]$')
    
    def handle_data(self, data):
        if self._accumulate_data:
            #print 'GOT DATA (%s)' % data
            if self.ignore_data.search(data):
                #print 'IGNORING'
                pass
            else:
                self._buffer = self._buffer + data
        return


class AllTheWebSearchPage(SearchPageBase):

    cookies = [
        ( 'PREF',
          'c=web:cwo=lang:aco=1:_lm=1014476174:c_hl=1:co=0:c_reswin=0:h=100:c_tip=0:c_sb=0:c_sm=0:ighits=2:l=any:cs=utf-8:qtf=y:c_clu=y:no=off',
          ),
        ]


    def newParser(self):
        return AllTheWebSearchResultsList()
    
    
class AllTheWebSearchEngine(SearchEngineBase):

    name = 'AllTheWeb'
    url_base = 'http://www.alltheweb.com'
    initial_search_pages = (
        '/search?cat=web&lang=english&query=%(searchTerms)s',
        '/search?cat=news&lang=english&query=%(searchTerms)s',
        )

    def __call__(self, searchTermText):
        """Return a sequence of AllTheWebSearchPage instances for
        the searchTermText.
        """
        #print '%s.__call__' % self.__class__.__name__
        query_terms = urllib.quote_plus(searchTermText)
        url_arguments = { 'searchTerms':query_terms }
        actions = []
        for initial_page in self.initial_search_pages:
            url_template = '%s%s' % (self.url_base, initial_page)
            url = url_template % url_arguments
            actions.append(AllTheWebSearchPage(url))
        return actions
    

def entryPoint():
    return { 'name':'AllTheWeb',
             'factory':AllTheWebSearchEngine,
             }


if __name__ == '__main__':
    data = open('/home/zope/WebAgents/sample_alltheweb_output.html', 'rt').read()
    
    parser = AllTheWebSearchResultsList()
    parser.feed(data)
    parser.close()

    print str(parser)
