#!/usr/bin/env python
#
# $Id: searchengines_google.py,v 1.3 2002/03/11 12:45:18 zope Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Parse Google search results.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: searchengines_google.py,v $',
    'rcs_id'       : '$Id: searchengines_google.py,v 1.3 2002/03/11 12:45:18 zope Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'WebAgents',
    'created'      : 'Mon, 31-Dec-2001 08:20:34 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: zope $',
    'version'      : '$Revision: 1.3 $',
    'date'         : '$Date: 2002/03/11 12:45:18 $',
}
try:
    __version__ = __rcs_info__['version'].split(' ')[1]
except:
    __version__ = '0.0'

#
# Import system modules
#
import pprint
import re
import string
import sys
import urllib

#
# Import Local modules
#
from SearchResults import SearchResultsList, SearchResultItem
from searchengines import SearchEngineBase, SearchPageBase

#
# Module
#

class GoogleSearchResultsList(SearchResultsList):

    parseResultExpr = re.compile(
        '<p><a href="?(?P<link>[^" >]*)[^>]*>(?P<title>.*?)</a>.*?<br>.*?<font[^>]*>(?P<description>.*)'
        )
    
    parseNextPageExpr = re.compile('<td><a href="?(?P<link>[^" ]*)"?.*><img src=')

    def feed(self, text):
        lines = text.split('\n')
        parser = self.parseResultExpr
        for line in lines:
            line = line.strip()
            if not line:
                continue
            if line[:3] == '<p>':
                # Line with link and description
                match = parser.search(line)
                #match = parser.match(line)
                if match:
                    params = match.groupdict()
                    #print 'MATCHED:',
                    #pprint.pprint(params)
                    #print 
                    new_item = SearchResultItem(params['link'],
                                                params['title'],
                                                params['description'])
                    self.append(new_item)
            elif line[:4] == '<td>':
                # Possibly a line with "next" page of results
                match = self.parseNextPageExpr.search(line)
                if match:
                    link = match.groupdict()['link']
                    self.addContinuation(link)
        return

    def addContinuation(self, link):
        url = GoogleSearchEngine.url_base + link
        self._continuation_links.append(GoogleSearchPage(url))
        return

class GoogleSearchPage(SearchPageBase):
    cookies = [
        ('PREF',
         'ID=2596116a018b7b68:LD=en:NR=100:NW=1:TM=993551980:LM=996694990',
         ),
        ]

    def newParser(self):
        return GoogleSearchResultsList()
        
    
class GoogleSearchEngine(SearchEngineBase):

    name = 'Google'

    url_base = 'http://www.google.com'
    initial_search_page = '/search?lr=lang_en&start=0&filter=0&q=%(searchTerms)s'

    search_page_factory = GoogleSearchPage

    def __call__(self, searchTermText):
        """Return a sequence of GoogleSearchPage instances for
        the searchTermText.
        """
        #print '%s.__call__' % self.__class__.__name__
        url_template = '%s%s' % (self.url_base, self.initial_search_page)
        query_terms = urllib.quote_plus(searchTermText)
        url_arguments = { 'searchTerms':query_terms }
        url = url_template % url_arguments
        return [ self.search_page_factory(url) ]
    


def entryPoint():
    return { 'name':'Google',
             'factory':GoogleSearchEngine,
             }


if __name__ == '__main__':
    data = open('sample_google_output.html', 'rt').read()
    
    parser = GoogleSearchResultsList()
    parser.feed(data)
    parser.close()

    print str(parser)
